from concurrent import futures

import grpc

from ll_sdk.config_api import ConfigApiClient
import llnw_pb2_grpc
from llnw_pb2 import *


def get_cl(base):
    return ConfigApiClient('apis.llnw.com', base.username, base.key)


def delivery_to_grpc(delivery):
    return DeliveryEntity(
        uuid=delivery['uuid'],
        isEnabled=delivery['isEnabled'],
        shortname=delivery['shortname'],
        status=delivery['status']['state'],
        sourceHostname=delivery['body']['sourceHostname'],
        publishedHostname=delivery['body']['publishedHostname'],
        sourceUrlPath=delivery['body']['sourceUrlPath'],
        publishedUrlPath=delivery['body']['publishedUrlPath'],
        protocolSets=[ProtocolSet(
            sourceProtocol=protocolSets['sourceProtocol'],
            publishedProtocol=protocolSets['publishedProtocol'],
            options=[ProtocolSetOptions(
                name=options['name'],
                parameters=options['parameters'],
                source=options['source'],
            ) for options in protocolSets['options']]
        ) for protocolSets in delivery['body']['protocolSets']]
    )


def delivery_from_grpc(delivery_entity: DeliveryEntity):
    return {
        "uuid": delivery_entity.uuid,
        "accounts": [
            {
                "shortname": delivery_entity.shortname
            }
        ],
        "body": {
            "description": "由 python API 提交",
            "protocolSets": [
                {
                    "options": [
                        {
                            "name": po.name,
                            "parameters": po.parameters,
                            "source": po.source
                        } for po in protocol.options
                    ],
                    "publishedProtocol": protocol.publishedProtocol,
                    "sourceProtocol": protocol.sourceProtocol
                } for protocol in delivery_entity.protocolSets
            ],
            "publishedHostname": delivery_entity.publishedHostname,
            "publishedUrlPath": delivery_entity.publishedUrlPath,
            "serviceKey": {
                "name": "delivery"
            },
            "serviceProfileName": "LLNW-Generic",
            "serviceProfileVersion": 2,
            "sourceHostname": delivery_entity.sourceHostname,
            "sourceUrlPath": delivery_entity.sourceUrlPath
        },
        "meta": {
            "allowManualDuplicate": True
        }
    }


class DeliveryServiceServicer(llnw_pb2_grpc.DeliveryServiceServicer):

    def ListDelivery(self, request, context):
        res = get_cl(request.base).list_delivery_service_instances(request.shortName).json()
        return DeliveryList(deliveries=[delivery_to_grpc(results) for results in res['results']])

    def ValidateDelivery(self, request, context):
        delivery_config = delivery_from_grpc(request.delivery)
        res = get_cl(request.base).validate_delivery_service_instance(delivery_config).json()
        return delivery_to_grpc(res)

    def CreateDelivery(self, request, context):
        delivery_config = delivery_from_grpc(request.delivery)
        res = get_cl(request.base).create_delivery_service_instance(delivery_config).json()
        return delivery_to_grpc(res)

    def GetDelivery(self, request, context):
        res = get_cl(request.base).get_delivery_service_instance(request.id).json()
        return delivery_to_grpc(res)

    def UpdateDelivery(self, request, context):
        delivery_config = delivery_from_grpc(request.delivery)
        res = get_cl(request.base).update_delivery_service_instance(delivery_config.uuid. delivery_config).json()
        return delivery_to_grpc(res)

    def DeleteDelivery(self, request, context):
        res = get_cl(request.base).delete_delivery_service_instance(request.id).json()
        return delivery_to_grpc(res)


if __name__ == '__main__':
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    llnw_pb2_grpc.add_DeliveryServiceServicer_to_server(DeliveryServiceServicer(), server)
    server.add_insecure_port('[::]:50052')
    server.start()
    server.wait_for_termination()
